"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const redux_1 = require("redux");
const lodash_1 = __importDefault(require("lodash"));
exports.loadingReducer = (state = {}, action) => {
    const { type } = action;
    const matches = /(.*)_(REQUEST|SUCCESS|FAILURE)/.exec(type);
    if (!matches)
        return state;
    const [, requestName, requestState] = matches;
    return Object.assign(Object.assign({}, state), { [requestName]: requestState === 'REQUEST' });
};
exports.errorReducer = (state = {}, action) => {
    const { type, error } = action;
    const matches = /(.*)_(REQUEST|FAILURE)/.exec(type);
    if (!matches)
        return state;
    const [, requestName, requestState] = matches;
    return Object.assign(Object.assign({}, state), { [requestName]: requestState === 'FAILURE' ? error : null });
};
exports.createLoadingSelector = (requestNames) => (state) => lodash_1.default(requestNames)
    .castArray()
    .some(type => lodash_1.default.get(state, `async.loading.${type.replace(/_(REQUEST|SUCCESS|FAILURE)/, '')}`));
exports.createErrorSelector = (requestNames) => (state) => lodash_1.default(requestNames)
    .castArray()
    .map(type => lodash_1.default.get(state, `async.errors.${type.replace(/_(REQUEST|SUCCESS|FAILURE)/, '')}`))
    .compact()
    .first() || null;
exports.createAsyncSelector = (requestNames) => {
    const getLoading = exports.createLoadingSelector(requestNames);
    const getError = exports.createErrorSelector(requestNames);
    return (state) => ({
        loading: getLoading(state),
        error: getError(state)
    });
};
exports.asyncReducer = redux_1.combineReducers({
    loading: exports.loadingReducer,
    errors: exports.errorReducer
});
